// Converts this component to a client-side component because it uses state
"use client";

import { urlFor } from "client";
import Image from "next/image";
import { useState } from "react";
import { BsChevronLeft, BsChevronRight } from "react-icons/bs";
import { images } from "types/sanity";

/**
 * @param params Takes and array of images as parameter
 * @returns The imagecarousel which is used in the Details-component aswell as in the homepage for the national market
 */

const ImageCarousel = ({ params }: { params: images }) => {
  // State for managing which image from the array that should be shown
  const [activeIndex, setActiveIndex] = useState<number>(0);

  // Sorting the images based on the img_number field and stores it
  const sortedImages: images | 0 = params.sort((a, b) => {
    if (a.img_number && b.img_number) {
      return a.img_number - b.img_number;
    } else {
      return 0;
    }
  });

  // Function that navigates to the previous image in the list
  const handlePrevClick = (): void => {
    setActiveIndex((activeIndex - 1 + params.length) % params.length);
  };

  // Function that navigates to the next image in the list
  const handleNextClick = (): void => {
    setActiveIndex((activeIndex + 1) % params.length);
  };

  return (
    <div className="flex flex-col justify-center items-center w-full px-1 bg-gray-100 h-full rounded-md">
      <div className="flex flex-col justify-center items-center w-full  h-full">
        <div className="flex justify-between items-center flex-col w-full h-full pb-5 ">
          <div className="flex flex-col justify-center items-center w-full h-full pt-5 lg:pt-2">
            {/* The div for the image itself. This must be set to postition: relative because we're using the "fill"-property in the image for dynamic sizing */}
            <div
              aria-labelledby="CarouselActiveContainer"
              className="relative justify-center w-full lg:w-4/5 h-72 xl:h-96 "
            >
              <Image
                aria-label="activeMain"
                className="object-contain shadow-lg lg:shadow-xl rounded-md"
                src={urlFor(sortedImages[activeIndex].asset._ref).url() || ""}
                alt={sortedImages[activeIndex].alt}
                fill
                priority // Sets the image to high priority so that it will preload and because they are part of the "Largest Contentful Paint (LCP)". Should be used according to next/image docs
              />
            </div>
            {/* The description below the image */}
            <div className="flex justify-center items-center font-semibold py-5 xl:text-lg  ">
              {sortedImages[activeIndex].img_description}
            </div>
            {params.length > 1 ? (
              <div className="flex flex-row justify-center items-center gap-6">
                <button
                  aria-label="Previous"
                  className="px-2  bg-white/75 cursor-pointer"
                  onClick={handlePrevClick}
                >
                  <BsChevronLeft className="h-6 w-6" />
                </button>
                <button
                  aria-label="Next"
                  className="px-2 bg-white/75 cursor-pointer"
                  onClick={handleNextClick}
                >
                  <BsChevronRight className="h-6 w-6" />
                </button>
              </div>
            ) : (
              <></>
            )}
          </div>
        </div>
        {/* If the image-array contains more than one image it will show the thumbnails below the image  */}
        {params.length > 1 ? (
          <>
            <div className=" w-full lg:w-full  	flex justify-center items-center  border-t border-black gap-5 lg:gap-2">
              <div className="w-[90%] lg:w-2/3  flex flex-wrap justify-center items-center ">
                {sortedImages.map((image, index) => {
                  return (
                    <div key={image._key} className=" py-2  self-center">
                      {image ? (
                        <div
                          className={`relative w-10 h-10 mx-2 rounded-sm col-span-1 py-5 cursor-pointer  ${
                            activeIndex === index
                              ? "border-2 border-warning"
                              : "border border-black"
                          }`}
                          onClick={() => setActiveIndex(index)}
                        >
                          <Image
                            aria-label="activeThumbnail"
                            className="object-contain"
                            src={urlFor(image.asset._ref).url() || ""}
                            fill
                            alt={image.alt}
                            priority
                          />
                        </div>
                      ) : (
                        <></>
                      )}
                    </div>
                  );
                })}
              </div>
            </div>
          </>
        ) : (
          <></>
        )}
      </div>
    </div>
  );
};

export default ImageCarousel;
