import Details from "app/components/Details";
import { Metadata } from "next";
import { getItemBySlug, getItemsByType } from "queries/service";
import { product } from "types/sanity";

// Exports a built in const that sets when the content of the page should revalidate, and if outdated refetched. In seconds so 3600 equals one hour
export const revalidate = 3600;

// Exports the built in function that can be used in combination with dynamic route segments to statically generate routes at build time instead of on-demand at request time.
export async function generateStaticParams() {
  const products: product[] = await getItemsByType("products");

  return products.map((product) => ({
    slug: product.slug,
  }));
}

// Sets the type for the page props. We have the [slug] which should be string
interface PageProps {
  params: {
    slug: string;
  };
}

/**
 * Exports a built in function that dynamically generates the metadata for this page. 
 * Because we are using the revalidate, and because Next automatically "dedupes" the requests to sanity we can run the same query multiple times without casuing many request 
 */
export async function generateMetadata({
  params,
}: PageProps): Promise<Metadata> {
  const res: product[] = await getItemBySlug(params.slug);
  return { title: res[0].title, description: res[0].details };
}

/**
 * @param params Because this is a page.tsx file it can fetch the dynamic route (which is in the [] in the route)
 * @returns The page for a specific product (detector), based on the users navigation
 */

const Page = async ({ params }: PageProps) => {
  // Fetching the data for the specific item based on the slug (the parameter the user has navigated to)
  const product: product[] = await getItemBySlug(params.slug);
  return (
    <div className="w-screen bg-light-bg">
      <Details params={product[0]} />
    </div>
  );
};

export default Page;
